//
//  INKCachedImageManager.m
//  Net Kit Sample
//
//  Created by iTech on 24.04.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "INKCachedImageManager.h"

@implementation INKCachedImageManager
@synthesize defaultImage = _defaultImage;

- (void)dealloc {
    [_imagesInfo release];
    [super dealloc];
}

- (id)init {
    self = [super init];
    if (self) {
        _imagesInfo = [[NSMutableDictionary alloc] init];
    }
    return self;
}


- (NSArray*)keys {
    return [[_imagesInfo allKeys] sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];
}

- (NSArray*)images {
    return [_imagesInfo allValues];
}

- (NSInteger)imagesCount {
    return [_imagesInfo count];
}

- (NSDictionary*)imagesInfo {
    return [[_imagesInfo copy] autorelease];
}


- (BOOL)addImage:(UIImage *)image withKey:(NSString *)key {
    return [self addImage:image withKey:key replace:YES];
}

- (BOOL)addImage:(UIImage *)image withKey:(NSString *)key replace:(BOOL)replace {
    if (!replace) {
        // Check if any image with the same key exist
        UIImage *existingImage = [_imagesInfo objectForKey:key];
        if (existingImage) {
            return NO;
        }
    }
    [_imagesInfo setObject:image forKey:key];
    return YES;
}

- (BOOL)addImages:(NSArray *)images forKeys:(NSArray *)keys {
    NSUInteger imagesCount = [images count];
    if (imagesCount!=[keys count]) {
        return NO;
    }
    BOOL allImagesSuccessfullyInserted = YES;
    for (NSInteger imageIndex = 0; imageIndex < imagesCount; imageIndex++) {
        UIImage *image = [images objectAtIndex:imageIndex];
        NSString *key = [keys objectAtIndex:imageIndex];
        BOOL imageSuccess = [self addImage:image withKey:key replace:YES];
        if (!imageSuccess) {
            allImagesSuccessfullyInserted = NO;
        }
    }
    return allImagesSuccessfullyInserted;
}

- (BOOL)removeImage:(UIImage *)image withKey:(NSString *)key {
    if (!key || key.length<1) {
        return NO;
    }
    if ([_imagesInfo objectForKey:key]) {
        [_imagesInfo removeObjectForKey:key];
        return YES;
    }
    return NO;
}

- (UIImage*)imageForKey:(NSString *)key {
    UIImage *image = [_imagesInfo objectForKey:key];
    if (!image) {
        return _defaultImage;
    }
    return image;
}

- (NSArray*)keysWithNULLImage {
    NSMutableArray *keysWithNULLImage = [[NSMutableArray alloc] init];
    [_imagesInfo enumerateKeysAndObjectsUsingBlock:^(id key, id obj, BOOL *stop) {
        if (obj == [NSNull null]) {
            [keysWithNULLImage addObject:key];
        }
    }];
    NSArray *resultArray = [keysWithNULLImage copy];
    [keysWithNULLImage release];
    return [resultArray autorelease];
}

- (void)clearCache {
    [_imagesInfo removeAllObjects];
}

@end
