//
//  INKImageLoader.h
//  Net Kit Sample
//
//  Created by iTech on 24.04.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "INKCachedImageManager.h"
#import "INKRequestsManager.h"



@interface INKImageLoader : INKCachedImageManager {
    INKRequestsManager *requestsManager;
    NSMutableDictionary *requestsInfo;
    
    NSURLRequestCachePolicy _requestsCachePolicy;
    NSTimeInterval _requestsTimeout;
}
@property (nonatomic, readonly) INKRequestsManager *requestsManager;

@property (nonatomic, assign) NSURLCacheStoragePolicy requestsCachePolicy;
@property (nonatomic, assign) NSTimeInterval requestsTimeout;

// Add request to queue.
- (void)downloadImageFromLink:(NSString*)link 
                       forKey:(NSString*)key;

// Add request to queue with operation block.
- (void)downloadImageFromLink:(NSString *)link 
                       forKey:(NSString *)key 
           withOperationBlock:(INKImageLoaderOperationBlock)block;

// Add request at first index in queue with operation block
- (void)downloadImportantImageFromLink:(NSString *)link 
                                forKey:(NSString *)key 
                    withOperationBlock:(INKImageLoaderOperationBlock)block;

// Set maximum downloading images. All adding images will be added to queue.
- (void)setMaximumImageDownloads:(NSInteger)downloadsLimit;

// Return maximum downloading images.
- (NSInteger)maximumImagesDownload;

// Place request to the first index in queue. If alredy downloding - do nothing.
- (void)accelerateDownloadingImageForKey:(NSString*)key;

// Return image for key. If no image finded return default image and add request for downloading.
- (UIImage*)imageForKey:(NSString *)key ifNotFindedDownloadFromLink:(NSString*)link;

// Return image for key. If no image finded return default image and add request for downloading with operation block.
- (UIImage*)imageForKey:(NSString *)key ifNotFindedDownloadFromLink:(NSString*)link 
     withOperationBlock:(INKImageLoaderOperationBlock)block;

// Return image for key. If no image finded return default image and add request for downloading with operation block.
//  If image request in queue then relace request at first index in queue.
- (UIImage*)imageForKey:(NSString *)key ifNotFindedDownloadFromLink:(NSString*)link 
     withOperationBlock:(INKImageLoaderOperationBlock)block 
    replaceAtFirstIndex:(BOOL)replaceAtFirstIndex;

// Return YES if image request in downloading.
- (BOOL)isImageForKeyLoadingNow:(NSString*)key;

// Return YES if image was already added to manager and waiting for downloading.
- (BOOL)isImageForKeyWaitingInQueue:(NSString*)key;

// Return YES if image waiting for downloading or downloading already.
- (BOOL)isImageForKeyWasAdded:(NSString*)key;

// Return YES if image for key is waiting for downloading.
- (BOOL)isImageForKeyWaitingInQueue:(NSString *)key;

- (BOOL)isImageForKeyWasDownloaded:(NSString*)key;

// Return download progress of image for key. Return 0.0 if image for key is not finded or request for image is in queue.
//  Return 1.0 if image already downloaded.
- (float)downloadProgressOfImageForKey:(NSString*)key;



@end
