//
//  INKRequestManager.h
//  Net Kit Sample
//
//  Created by Vladlex on 22.04.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "INKHelper.h"


@class INKURLRequest;

// NSURLConnectinoDelegate using only if IOS 4 is actual version
@interface INKRequestsManager: NSObject <NSURLConnectionDataDelegate>

{
    BOOL isIOSVersion5orHigher;
    
    NSMutableArray *_processingRequests;
    NSMutableArray *_queueRequests;
        
    NSUInteger _processingRequestsLimit;
    
    NSMutableArray *connections;
}

// Maximum of processing requests. All other requests are adding to queue ath the end.
// Default is 0 (No limit).
@property (nonatomic, assign) NSUInteger processingRequestsLimit;


// This methods returns you actual requests arrays. They are immutanle copy of original arrays.
- (NSArray*)processingRequests;
- (NSArray*)queueRequests;

// Use this method to get arrays count without Mutable -> Immutable transformation
- (NSInteger)processingRequestsCount;
- (NSInteger)queueLength;

/* --- ===  Requests operations  === --- */

// Normal request adding. If request can't be processed right now he adding to the end of queue.
- (void)addRequest:(INKURLRequest*)request
withCompletionBlock:(INKURLRequestOperationBlock)completionBlock;

// Important request adding. If request can't be processed right now he adding at the first index of queue
- (void)addImportantRequest:(INKURLRequest*)request
        withCompletionBlock:(INKURLRequestOperationBlock)completionBlock;

// Canceling request even he already processing.
- (void)cancelRequest:(NSURLRequest*)request;

// Check if any request can be added to processing requests list.
- (BOOL)requestsCanBeAddedToProcessing;

// Check if current request is waiting for server respinse now
- (BOOL)isRequestIsProcessing:(INKURLRequest*)request;

// Check if current request is in queue now
- (BOOL)isRequestInQueue:(INKURLRequest*)request;

// Replace request to the first place in queue.
- (void)replaceRequestAtFirstPlace:(INKURLRequest*)request;

// Cancel all requests
- (void)cancelAllRequests;

// Cancel all requests in queue but keep processing requests
- (void)clearQueue;

// Remove all processings requests
- (void)cancelProcessingRequests;

@end
