//
//  INKRequestManager.m
//  Net Kit Sample
//
//  Created by Vladlex on 22.04.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "INKRequestsManager.h"

#import "INKHelper.h"
#import "INKURLConnection.h"
#import "INKURLRequest.h"

@interface INKRequestsManager ()

- (void)addRequestToQueue:(INKURLRequest *)request atIndex:(NSInteger)index;
- (void)startRequest:(INKURLRequest*)request;

- (void)addRequest:(INKURLRequest *)request withCompletionBlock:(INKURLRequestOperationBlock)completionBlock isImportant:(BOOL)isImportant;

- (INKURLConnection*)connectionForRequest:(INKURLRequest*)request;

- (void)removeConnection:(INKURLConnection*)connection;
- (void)startNextRequest;

@end

@implementation INKRequestsManager
@synthesize processingRequestsLimit = _processingRequestsLimit;

- (void)dealloc {
    [_processingRequests release];
    [_queueRequests release];
    [connections release];
    
    [super dealloc];
}

- (id)init {
    self = [super init];
    if (self) {
        
        // 1. Define ios version.
        float iosVersion = [[[UIDevice currentDevice] systemVersion] floatValue];
        if (iosVersion < 5.0) {
            isIOSVersion5orHigher = NO;
        }
        else {
            isIOSVersion5orHigher = YES;
        }
        
        _processingRequestsLimit = 5;
        _processingRequests = [[NSMutableArray alloc] init];
        _queueRequests = [[NSMutableArray alloc] init];
        
        connections = [[NSMutableArray alloc] init];
    }
    return self;
}

#pragma mark Accessors

- (NSArray*)processingRequests {
    return [NSArray arrayWithArray:_processingRequests];
}

- (NSInteger)processingRequestsCount {
    return _processingRequests.count;
}

- (NSArray*)queueRequests {
    return [NSArray arrayWithArray:_queueRequests];
}

- (NSInteger)queueLength {
    return _queueRequests.count;
}

#pragma mark Requests operations

- (void)startNextRequest {
    if (_queueRequests.count>0) {
        INKURLRequest *nextRequest = [_queueRequests objectAtIndex:0];
        [_queueRequests removeObjectAtIndex:0];
        [self startRequest:nextRequest];
    }
}

- (void)addRequest:(INKURLRequest *)request withCompletionBlock:(INKURLRequestOperationBlock)completionBlock isImportant:(BOOL)isImportant {
    INKURLRequest *preparedRequest = (INKURLRequest*)request;
    if (![request isKindOfClass:[INKURLRequest class]]) {
        if ([request isKindOfClass:[NSURLRequest class]]) {
            preparedRequest = [INKURLRequest inkRequestCopy:request];
        }
    }
    [preparedRequest setRequestOperationBlock:completionBlock];
    
    if ([self requestsCanBeAddedToProcessing]) {
        [self startRequest:preparedRequest];
    }
    else {
        if (isImportant) {
            [_queueRequests insertObject:preparedRequest atIndex:0];
        }
        else {
            [_queueRequests addObject:preparedRequest];
        }
    }
}

- (void)addRequest:(INKURLRequest *)request withCompletionBlock:(INKURLRequestOperationBlock)completionBlock {
    [self addRequest:request withCompletionBlock:completionBlock isImportant:NO];
}

- (void)addImportantRequest:(INKURLRequest *)request withCompletionBlock:(INKURLRequestOperationBlock)completionBlock {
    [self addRequest:request withCompletionBlock:completionBlock isImportant:YES];
}

- (void)cancelRequest:(INKURLRequest *)request {
    // If request in queue
    NSInteger queueIndex = [_queueRequests indexOfObjectIdenticalTo:request];
    if (queueIndex!=NSNotFound) {
        [_queueRequests removeObjectAtIndex:queueIndex];
        return;
    }
    INKURLConnection *connection = [self connectionForRequest:request];
    if (connection) {
        [connection cancel];
        [self removeConnection:connection];
    }
}

- (void)cancelAllRequests {
    [self clearQueue];
    [self cancelProcessingRequests];
}

- (void)clearQueue {
    [_queueRequests removeAllObjects];
}

- (void)cancelProcessingRequests {
    for (INKURLConnection *connection in connections) {
        [connection cancel];
    }
    
    /*    NSInteger queueIndex = [_queueRequests indexOfObjectIdenticalTo:request];
     if (queueIndex!=NSNotFound) {
     [_queueRequests removeObjectAtIndex:queueIndex];
     return;
     }
     INKURLConnection *connection = [self connectionForRequest:request];
     if (connection) {
     [connection cancel];
     [self removeConnection:connection];
     }

     
     */
    [_processingRequests removeAllObjects];
}

- (void)replaceRequestAtFirstPlace:(INKURLRequest *)request {
    NSInteger startRequestIndex = [_queueRequests indexOfObjectIdenticalTo:request];
    if (startRequestIndex == 0 || startRequestIndex == NSNotFound) {
        // Request already at first place or does not exist or already processing
        return;
    }
    [_queueRequests removeObject:request];
    [_queueRequests insertObject:request atIndex:0];
}

#pragma mark Other Public methods

- (BOOL)requestsCanBeAddedToProcessing {
    if (_processingRequestsLimit==0) {
        return YES;
    }
    if (_processingRequests.count >= _processingRequestsLimit) {
        return NO;
    }
    return YES;
}

- (BOOL)isRequestIsProcessing:(INKURLRequest *)request {
    NSInteger index = [_processingRequests indexOfObjectIdenticalTo:request];
    return (index!=NSNotFound) ? YES : NO;
}

- (BOOL)isRequestInQueue:(INKURLRequest *)request {
    NSInteger index = [_queueRequests indexOfObjectIdenticalTo:request];
    return (index!=NSNotFound) ? YES : NO;
}

#pragma mark Private methods

- (void)addRequestToQueue:(INKURLRequest *)request atIndex:(NSInteger)index {
    EasyLog(@"Adding request to queue at index %d", index);
    if (index > _queueRequests.count) {
        [_queueRequests addObject:request];
        return;
    }
    [_queueRequests insertObject:request atIndex:index];
}

- (void)startRequest:(INKURLRequest *)request {
    if ([_processingRequests indexOfObjectIdenticalTo:request]==NSNotFound) {
        [_processingRequests addObject:request];
    }
    INKURLConnection *urlConnection = (id)[[INKURLConnection alloc]initWithRequest:request delegate:self startImmediately:NO];
    [connections addObject:urlConnection];
    [urlConnection scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
    [request performBlockWithCompletion:NO cause:INKURLRequestOperationBlockCauseStartLoading];
    [urlConnection start];
    [urlConnection release];
}

- (INKURLConnection*)connectionForRequest:(INKURLRequest *)request {
    for (INKURLConnection *connection in connections) {
        if ([connection isKindOfClass:[INKURLConnection class]]) {
            // Do not use "isEqual", only pointers equaling
            if (connection.sendingRequest == request) {
                return connection;
            }
        }
    }
    return nil;
}

- (void)removeConnection:(INKURLConnection *)connection {
    INKURLRequest *request = [connection sendingRequest];
    [connections removeObject:connection];
    [_processingRequests removeObject:request];
    
    [self startNextRequest];
}



#pragma mark NSURLConnectionDelegate

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
    INKURLRequest *connectionRequest = [(INKURLConnection*)connection sendingRequest];
    connectionRequest.response = response;
    [connectionRequest performBlockWithCompletion:NO cause:INKURLRequestOperationBlockCauseRecieveResponse];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
    INKURLRequest *connectionRequest = [(INKURLConnection*)connection sendingRequest];
    [connectionRequest performBlockWithCompletion:YES cause:INKURLRequestOperationBlockCauseFinishLoading];
    [self removeConnection:(INKURLConnection*)connection];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    INKURLRequest *connectionRequest = [(INKURLConnection*)connection sendingRequest];
    [connectionRequest.incomingData appendData:data];
    [connectionRequest performBlockWithCompletion:NO cause:INKURLRequestOperationBlockCauseDataPartLoaded];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
    INKURLRequest *connectionRequest = [(INKURLConnection*)connection sendingRequest];
    connectionRequest.connectionError = error;
    [connectionRequest performBlockWithCompletion:NO cause:INKURLRequestOperationBlockCauseConnectionError];
    [self removeConnection:(INKURLConnection*)connection];
}

- (void)connection:(NSURLConnection *)connection didSendBodyData:(NSInteger)bytesWritten totalBytesWritten:(NSInteger)totalBytesWritten totalBytesExpectedToWrite:(NSInteger)totalBytesExpectedToWrite {
    INKURLRequest *connectionRequest = [(INKURLConnection*)connection sendingRequest];
    connectionRequest.totalBytesExpectedToWrite = [NSNumber numberWithInteger:totalBytesExpectedToWrite];
    connectionRequest.totalBytesSended = [NSNumber numberWithInteger:totalBytesWritten];
    connectionRequest.sendedLastPacketBytes = [NSNumber numberWithInteger:bytesWritten];
    [connectionRequest performBlockWithCompletion:NO cause:INKURLRequestOperationBlockCauseDataPartUploaded];
}


@end
