//
//  INKURLRequest.m
//  Net Kit Sample
//
//  Created by Vladlex on 22.04.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//


#import "INKURLRequest.h"

@interface INKURLRequest () 

- (NSString*)methodNameForMethodType:(INKURLRequestMethodType)methodType;

@end


@implementation INKURLRequest
@synthesize requestOperationBlock = _requestOperationBlock;
@synthesize userInfo = _userInfo;
@synthesize incomingData = _incomingData;
@synthesize response = _response;
@synthesize connectionError = _connectionError;
@synthesize totalBytesSended = _totalBytesSended, totalBytesExpectedToWrite = _totalBytesExpectedToWrite, sendedLastPacketBytes = _sendedLastPacketBytes;

- (void)dealloc {
    [_userInfo release];
    [_connectionError release];
    [_response release];
    [_incomingData release];
    [_totalBytesExpectedToWrite release];
    [_totalBytesSended release];
    [_sendedLastPacketBytes release];
    [super dealloc];
}

- (id)initWithURL:(NSURL *)URL cachePolicy:(NSURLRequestCachePolicy)cachePolicy timeoutInterval:(NSTimeInterval)timeoutInterval {
    self = [super initWithURL:URL 
                  cachePolicy:cachePolicy 
              timeoutInterval:timeoutInterval];
    if (self) {
        _incomingData = [[NSMutableData alloc]init];
        _response = nil;
        _connectionError = nil;
        _userInfo = nil;
    }
    return self;
}

+ (id)requestWithURL:(NSURL *)URL cachePolicy:(NSURLRequestCachePolicy)cachePolicy timeoutInterval:(NSTimeInterval)timeoutInterval {
    return [super requestWithURL:URL cachePolicy:cachePolicy timeoutInterval:timeoutInterval];
}

- (float)downloadProgress {
    NSDictionary *httpFields = [self responseHTTPHeaderFields];
    if (httpFields) {
        NSNumber *promisedDataLengthNum = [httpFields objectForKey:@"Content-Length"];
        NSInteger promisedDataLength = promisedDataLengthNum.integerValue;
        if (promisedDataLength==0) {
            return 0.0;
        }
        return (float)_incomingData.length/promisedDataLength;
    }
    return 0.0;
}

- (void)setMethod:(INKURLRequestMethodType)method {
    [self setValue:[self methodNameForMethodType:method] forHTTPHeaderField:@"Method"];
}

+ (id)inkRequestCopy:(NSURLRequest *)request {
    id newRequest = [[self alloc] initWithURL:request.URL cachePolicy:request.cachePolicy timeoutInterval:request.timeoutInterval];
    return [newRequest autorelease];
}

- (void)performBlockWithCompletion:(BOOL)completion cause:(INKURLRequestOperationBlockCause)cause {
    if (_requestOperationBlock) {
        [[NSOperationQueue mainQueue] addOperationWithBlock:^{
            _requestOperationBlock(completion, cause);
        }];
    }
}

- (NSDictionary*)responseHTTPHeaderFields {
    if (_response!=nil && [_response isKindOfClass:[NSHTTPURLResponse class]]) {
        return [(NSHTTPURLResponse*)_response allHeaderFields];
    }
    return nil;
}

#pragma mark Private methods

- (NSString*)methodNameForMethodType:(INKURLRequestMethodType)methodType {
    switch (methodType) {
        case INKURLRequestMethodTypeDELETE:
            return @"DELETE";
            break;
        case INKURLRequestMethodTypeGET:
            return @"GET";
            break;
        case INKURLRequestMethodTypePUT:
            return @"PUT";
            break;
        case INKURLRequestMethodTypeHEAD:
            return @"HEAD";
            break;
        case INKURLRequestMethodTypePOST:
            return @"POST";
            break;
        case INKURLRequestMethodTypeTRACE:
            return @"TRACE";
            break;
        default:
            return @"POST";
            break;
    }
}

@end
