//
//  ITCHBigImagesViewController.m
//  Net Kit Sample
//
//  Created by iTech on 24.04.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "ITCHBigImagesViewController.h"

#import "INKImageLoader.h"

@interface ITCHBigImagesViewController ()
- (void)configureCell:(UITableViewCell*)cell atIndexPath:(NSIndexPath*)indexPath;
- (void)reloadImages:(id)sender;

@end

@implementation ITCHBigImagesViewController

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
        imageLoader = [[INKImageLoader alloc] init];
        imageLoader.defaultImage = [UIImage imageNamed:@"640px_no_image.png"];
        imageLoader.requestsCachePolicy = NSURLRequestReloadIgnoringCacheData;
        [imageLoader setMaximumImageDownloads:2];
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.title = @"Big images";
    self.tableView.backgroundColor = [UIColor blackColor];
    self.tableView.rowHeight = self.view.bounds.size.width;
    self.tableView.autoresizingMask = (UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth);
    UIBarButtonItem *reloadItem = [[UIBarButtonItem alloc] 
                                   initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh 
                                   target:self 
                                   action:@selector(reloadImages:)];
    self.navigationItem.rightBarButtonItem = reloadItem;
    [reloadItem release];

    // Uncomment the following line to preserve selection between presentations.
    // self.clearsSelectionOnViewWillAppear = NO;
 
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    // self.navigationItem.rightBarButtonItem = self.editButtonItem;
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    [imageLoader clearCache];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

- (void)reloadImages:(id)sender {
    [imageLoader clearCache];
    [self.tableView reloadData];
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return 6;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellIdentifier] autorelease];
        
        UILabel *overImageLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, 200.0, 27.0)];
        overImageLabel.textAlignment = UITextAlignmentCenter;
        overImageLabel.backgroundColor = [UIColor clearColor];
        overImageLabel.tag = 101;
        overImageLabel.font = [UIFont systemFontOfSize:20.0];
        overImageLabel.textColor = [UIColor lightGrayColor];
        [overImageLabel setShadowColor:[UIColor blackColor]];
        [overImageLabel setShadowOffset:CGSizeMake(0, -1)];
        [cell.imageView addSubview:overImageLabel];
        [overImageLabel release];
        
        UIProgressView *progressView = [[UIProgressView alloc] initWithProgressViewStyle:UIProgressViewStyleBar];
        progressView.frame = CGRectMake(10.0, 10.0, cell.imageView.bounds.size.width-20.0, 27.0);
        progressView.progress = 0.0;
        progressView.tag = 102;
        [cell.imageView addSubview:progressView];
        [progressView release];
        
        UIActivityIndicatorView *activityIndicator = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
        activityIndicator.tag = 103;
        [cell.imageView addSubview:activityIndicator];
        [activityIndicator release];
        
    }
    [self configureCell:cell atIndexPath:indexPath];
    return cell;
}

- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath {
    NSString *aKey = [NSString stringWithFormat:@"%d", indexPath.row];
    NSString *link = [NSString stringWithFormat:@"http://dl.dropbox.com/u/69395379/640px_images/640px_test%d.png", indexPath.row];
    cell.imageView.image = imageLoader.defaultImage;
    UILabel *imageLabel = (UILabel*)[cell.imageView viewWithTag:101];
    UIProgressView *progressView = (UIProgressView*)[cell.imageView viewWithTag:102];
    UIActivityIndicatorView *activityIndicator = (UIActivityIndicatorView*)[cell.imageView viewWithTag:103];
    
    UIImage *image = [imageLoader imageForKey:aKey 
                  ifNotFindedDownloadFromLink:link 
                           withOperationBlock:^(BOOL finished, NSData *data, NSString *key, float progress, NSError *error) {
                               UITableViewRowAnimation animation = UITableViewRowAnimationNone;
                               if (finished) {
                                   animation = UITableViewRowAnimationFade;
                                   
                                   EasyLog(@"Image for key %@ from %@ downloaded", key, link);
                                   
                               }
                               [self.tableView reloadRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:animation];
                           } 
                          replaceAtFirstIndex:YES];
    cell.imageView.image = image;
    
    imageLabel.text = [NSString stringWithFormat:@"Image %d", indexPath.row];
    [UIView animateWithDuration:0.0 animations:^{
        progressView.progress = 0.0;
    }];
    if ([imageLoader isImageForKeyLoadingNow:aKey]) {
        [activityIndicator stopAnimating];
        progressView.hidden = NO;
        imageLabel.hidden = NO;
        progressView.progress = [imageLoader downloadProgressOfImageForKey:aKey];
    }
    if ([imageLoader isImageForKeyWaitingInQueue:aKey]) {
        
        if (![activityIndicator isAnimating]) {
            [activityIndicator startAnimating];
        }
        imageLabel.hidden = NO;
        progressView.hidden = YES;
        [UIView animateWithDuration:0.0 animations:^{
            progressView.progress = 0.0;
        }
         ];
    }
    if ([imageLoader isImageForKeyWasDownloaded:aKey]) {
        [activityIndicator stopAnimating];
        progressView.hidden = YES;
        imageLabel.hidden = YES;
    }
}

- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath {
    UILabel *imageLabel = (UILabel*)[cell.imageView viewWithTag:101];
    CGPoint labelCenter = cell.imageView.center;
    labelCenter.y = cell.imageView.bounds.size.height / 4 * 2.5;
    imageLabel.center = labelCenter;
    
    UIProgressView *progressView = (UIProgressView*)[cell.imageView viewWithTag:102];
    CGRect progressFrame = [progressView frame];
    progressFrame.origin.y = cell.imageView.bounds.size.height - progressView.bounds.size.height - 10.0;
    progressFrame.origin.x = 10.0;
    progressFrame.size.width = cell.imageView.bounds.size.width-20.0;
    progressView.frame = progressFrame;
    
    UIActivityIndicatorView *activityIndicator = (UIActivityIndicatorView*)[cell.imageView viewWithTag:103];
    CGPoint activityIndicatorCenter = cell.imageView.center;
    activityIndicatorCenter.y = progressFrame.origin.y - 30.0;
    activityIndicator.center = activityIndicatorCenter;
}

/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/

/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/

/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
}
*/

/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Navigation logic may go here. Create and push another view controller.
    /*
     <#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName:@"<#Nib name#>" bundle:nil];
     // ...
     // Pass the selected object to the new view controller.
     [self.navigationController pushViewController:detailViewController animated:YES];
     [detailViewController release];
     */
}

@end
