//
//  HTTPHeaderFieldsViewController.m
//  Net Kit Sample
//
//  Created by iTech on 24.04.12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "ITCHHTTPHeaderFieldsViewController.h"

#import "INKit.h"

@interface ITCHHTTPHeaderFieldsViewController ()
- (void)configureWithNewHeaderFields:(NSDictionary*)headerFields;
- (void)configureCell:(UITableViewCell*)cell atIndexPath:(NSIndexPath*)indexPath;
- (void)sendRequestAction:(id)sender;
- (void)sendRequestWithLink:(NSString*)link;
@end

@implementation ITCHHTTPHeaderFieldsViewController

- (void)dealloc {
    [requestsManager release];
    [httpHeadersTableView release];
    [responseHTTPHeaderFields release];
    [textField release];
    
    [super dealloc];
}

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        responseHTTPHeaderFields = [[NSArray alloc] init];
        isLoading = NO;
        requestsManager = [[INKRequestsManager alloc] init];
    }
    return self;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation {
    return YES;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = @"HTTP Headers";
    
    httpHeadersTableView = [[UITableView alloc] initWithFrame:self.view.bounds style:UITableViewStyleGrouped];
    httpHeadersTableView.dataSource = self;
    httpHeadersTableView.delegate = self;
    httpHeadersTableView.rowHeight = 48.0;
    httpHeadersTableView.autoresizingMask = (UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleHeight);
    
    [self.view addSubview:httpHeadersTableView];
    
    UIView *headerView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, self.view.bounds.size.width, 46.0)];
    textField= [[UITextField alloc] initWithFrame:CGRectMake(10.0, 10.0, self.view.bounds.size.width - 100.0, 26.0)];
    textField.borderStyle = UITextBorderStyleRoundedRect;
    textField.text = @"http://yandex.ru";
    textField.font = [UIFont systemFontOfSize:[UIFont systemFontSize]+1.0];
    textField.contentMode = UIViewContentModeCenter;
    textField.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    [headerView addSubview:textField];
    UIButton *sendRequestButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
    sendRequestButton.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin;
    sendRequestButton.frame = CGRectMake(CGRectGetMaxX(textField.frame)+10.0, textField.frame.origin.y, self.view.bounds.size.width - textField.bounds.size.width - 30.0, 26.0);
    [sendRequestButton setTitle:@"Ask" forState:UIControlStateNormal];
    [sendRequestButton addTarget:self action:@selector(sendRequestAction:) forControlEvents:UIControlEventTouchUpInside];
    [headerView addSubview:sendRequestButton];
    headerView.backgroundColor = [[UIColor blackColor] colorWithAlphaComponent:0.5];
    httpHeadersTableView.tableHeaderView = headerView;
    [headerView release];
}

- (void)configureWithNewHeaderFields:(NSDictionary *)headerFields {
    if (responseHTTPHeaderFields) {
        [responseHTTPHeaderFields release];
    }
    NSArray *sortedKeys = [[headerFields allKeys] sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];
    NSMutableArray *newHeaderFields = [[NSMutableArray alloc] init];
    for (NSString *key in sortedKeys) {
        NSDictionary *headerKeyAndValue = [NSDictionary dictionaryWithObjectsAndKeys:
                                           key, @"header",
                                           [headerFields objectForKey:key], @"value",
                                           nil];
        [newHeaderFields addObject:headerKeyAndValue];
    }
    
    responseHTTPHeaderFields = newHeaderFields;

    [httpHeadersTableView reloadSections:[NSIndexSet indexSetWithIndex:0] withRowAnimation:UITableViewRowAnimationFade];
}

- (void)sendRequestAction:(id)sender {
    if (isLoading) {
        return;
    }
    NSString *linkString = [textField text];
    if (linkString!=nil && linkString.length>0) {
        [self sendRequestWithLink:linkString];
    }
}

- (void)sendRequestWithLink:(NSString *)link {
    isLoading = YES;
    [httpHeadersTableView reloadSections:[NSIndexSet indexSetWithIndex:0] withRowAnimation:UITableViewRowAnimationLeft];

    
    NSURL *url = [NSURL URLWithString:link];
    INKURLRequest *request = [INKURLRequest requestWithURL:url];
    request.timeoutInterval = 5.0;
    [requestsManager addRequest:request withCompletionBlock:^(BOOL finished, INKURLRequestOperationBlockCause cause) {
        isLoading = NO;

        if (cause == INKURLRequestOperationBlockCauseRecieveResponse) {
            NSDictionary *response = [request responseHTTPHeaderFields];
            [self configureWithNewHeaderFields:response];
            // We don't need in request anymore, so cancel him.
            [requestsManager cancelRequest:request];
            return;
        }
        if (cause == INKURLRequestOperationBlockCauseConnectionError) {
            NSError *error = request.connectionError;
            NSDictionary *errorInfo = [NSDictionary dictionaryWithObjectsAndKeys:
                                       [NSNumber numberWithInteger:error.code].stringValue, @"Error code",
                                       error.localizedDescription, @"Error description",
                                       nil];
            [self configureWithNewHeaderFields:errorInfo];
//            [requestManager cancelRequest:request];
        }
    }];
}

#pragma mark Table view data source

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (isLoading) {
        return 1;
    }
    return responseHTTPHeaderFields.count;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue2 reuseIdentifier:cellIdentifier] autorelease];
        UIActivityIndicatorView *indicator = [[[UIActivityIndicatorView alloc]initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray] autorelease];
        indicator.hidesWhenStopped = YES;
        cell.accessoryView = indicator;
        cell.detailTextLabel.font = [UIFont systemFontOfSize:13.0];
        cell.detailTextLabel.numberOfLines = 0;
        cell.detailTextLabel.lineBreakMode = UILineBreakModeTailTruncation;
        cell.textLabel.font = [UIFont systemFontOfSize:14.0];
        cell.textLabel.numberOfLines = 0;
        cell.textLabel.lineBreakMode = UILineBreakModeTailTruncation;
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
    }
    [self configureCell:cell atIndexPath:indexPath];
    return cell;
}

- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath {
    UIActivityIndicatorView *indicator = (UIActivityIndicatorView*)cell.accessoryView;
    if (isLoading) {
        cell.detailTextLabel.text = @"";
        cell.textLabel.text = @"Loading...";
        [indicator startAnimating];
        return;
    }
    
    // Well, it's bad to sort dictionary keys every time, when table view need data for some cells.
    // I user it only for a sample.
    [indicator stopAnimating];
    NSDictionary *headerFieldInfo = [responseHTTPHeaderFields objectAtIndex:indexPath.row];
    NSString *httpHeaderField = [headerFieldInfo objectForKey:@"header"];
    cell.textLabel.text = httpHeaderField;
    cell.detailTextLabel.text = [headerFieldInfo objectForKey:@"value"];  
}

- (float)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    if (isLoading) {
        return 40.0;
    }
    NSDictionary *headerFieldInfo = [responseHTTPHeaderFields objectAtIndex:indexPath.row];
    NSString *detailedText = [headerFieldInfo objectForKey:@"header"];
    NSString *text = [headerFieldInfo objectForKey:@"value"];
    CGSize detailedTextLabelSize = CGSizeMake(60.0, 1000.0);
    detailedTextLabelSize = [detailedText sizeWithFont:[UIFont systemFontOfSize:13.0] constrainedToSize:detailedTextLabelSize];
    CGSize textLabelSize = CGSizeMake(self.view.bounds.size.width - detailedTextLabelSize.width - 150.0, 1000);
    textLabelSize = [text sizeWithFont:[UIFont systemFontOfSize:14.0] constrainedToSize:textLabelSize];
    float maxHeight = MAX(detailedTextLabelSize.height, textLabelSize.height);
    return maxHeight + 10.0;
}

 
@end
